<?php
/**
 * Beaver Builder providers
 *
 * @package   WP Grid Builder - Beaver Builder
 * @author    Loïc Blascos
 * @copyright 2019-2022 Loïc Blascos
 */

namespace WP_Grid_Builder_Beaver_Builder\Includes;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Providers
 *
 * @class WP_Grid_Builder_Beaver_Builder\Includes\Providers
 * @since 1.0.0
 */
final class Providers extends Observer {

	/**
	 * Holds default Beaver Builder providers
	 *
	 * @since 1.0.0
	 * @access public
	 * @var array
	 */
	public $defaults = [
		'post-grid'       => __NAMESPACE__ . '\Providers\Post_Grid',
		'post-slider'     => __NAMESPACE__ . '\Providers\Post_Slider',
		'post-carousel'   => __NAMESPACE__ . '\Providers\Post_Carousel',
		'woocommerce'     => __NAMESPACE__ . '\Providers\WooCommerce',
		'product-grid'    => __NAMESPACE__ . '\Providers\Product_Grid',
		'pp-content-grid' => __NAMESPACE__ . '\Providers\PP_Content_Grid',
	];

	/**
	 * Constructor
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function __construct() {

		parent::__construct();

		add_filter( 'wp_grid_builder_beaver_builder/providers', [ $this, 'register' ] );

	}

	/**
	 * Add default providers
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param array $providers Holds providers.
	 * @return array
	 */
	public function register( $providers ) {

		return array_merge( $this->defaults, $providers );

	}

	/**
	 * Handle provider before render
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param string       $name Provider name.
	 * @param array|string $attr Provider attributes.
	 */
	public function maybe_handle( $name, $attr ) {

		$module = $this->get_module( $name, $attr );

		if ( ! $module ) {
			return;
		}

		$instance = $this->instantiate( $name, $module );

		if ( ! $instance ) {
			return;
		}

		$instance->_before_render();

	}

	/**
	 * Handle provider after render
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param string       $name Provider name.
	 * @param array|string $attr Provider attributes.
	 */
	public function maybe_render( $name, $attr ) {

		$module = $this->get_module( $name, $attr );

		if ( ! $module ) {
			return;
		}

		$instance = Container::get( $module->id );

		if ( ! $instance ) {
			return;
		}

		$this->render_module( $name, $attr );
		$instance->_after_render();

	}

	/**
	 * Get module attributes
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param string $name Module name.
	 * @param array  $attr Module attributes.
	 * @return false|object
	 */
	public function get_module( $name, $attr ) {

		if ( ! $this->is_provider( $name ) ) {
			return false;
		}

		if ( ! isset( $attr->settings, $attr->node ) ) {
			return false;
		}

		$module     = clone $attr->settings;
		$module->id = 'bb-module-' . $attr->node;

		if ( empty( $attr->settings->wpgb ) ) {
			$module->settings = [ 'post_id' => \FLBuilderModel::get_post_id() ];
		} else {
			$module->settings = $attr->settings->wpgb;
		}

		return $module;

	}

	/**
	 * Render module shortcode
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param string $name Module name.
	 * @param array  $attr Module attributes.
	 */
	public function render_module( $name, $attr ) {

		if ( 'woocommerce' !== $name ) {
			return;
		}

		// We need to render the WooCommerce shortcode at the same time as the module rendering.
		// Otherwise, Beaver Builder will render it too late (the query will be generated after the module is rendered 'fl_builder_after_render_module').
		$content = preg_replace_callback(
			'/<[^>]*class="[^"]*\bfl-node-' . $attr->node . '\b[^"]*"[^>]+?>(?:[^$]+?)<\/div>/mi',
			function( $match ) {
				return do_shortcode( $match[0] );
			},
			ob_get_clean()
		);

		ob_start();
		echo $content; // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped

	}

	/**
	 * Get provider name
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param string $name Provider name.
	 * @return false|string
	 */
	public function is_provider( $name ) {

		$providers = apply_filters( 'wp_grid_builder_beaver_builder/providers', [] );

		if ( ! isset( $providers[ $name ] ) || ! class_exists( $providers[ $name ] ) ) {
			return false;
		}

		return $name;

	}

	/**
	 * Instantiate provider
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param string $provider Provider name.
	 * @param array  $module   Provider module.
	 * @return false|Container
	 */
	public function instantiate( $provider, $module ) {

		if ( ! apply_filters( 'wp_grid_builder_beaver_builder/handle_module', false, $module->id ) ) {
			return false;
		}

		$providers = apply_filters( 'wp_grid_builder_beaver_builder/providers', [] );

		return Container::add( $module->id, new $providers[ $provider ]( $module, $module->settings ) );

	}
}
