<?php
/**
 * Render modules
 *
 * @package   WP Grid Builder - Beaver Builder
 * @author    Loïc Blascos
 * @copyright 2019-2022 Loïc Blascos
 */

namespace WP_Grid_Builder_Beaver_Builder\Includes;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Render
 *
 * @class WP_Grid_Builder_Beaver_Builder\Includes\Render
 * @since 1.0.0
 */
final class Render {

	/**
	 * Holds Beaver Builder modules to handle
	 *
	 * @since 1.0.0
	 * @access protected
	 * @var array
	 */
	protected $modules = [];

	/**
	 * Constructor
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function __construct() {

		add_filter( 'wp_grid_builder_beaver_builder/handle_module', [ $this, 'handle_module' ], 10, 2 );
		add_filter( 'wp_grid_builder/frontend/register_scripts', [ $this, 'register_script' ] );
		add_filter( 'wp_grid_builder/frontend/add_inline_script', [ $this, 'inline_script' ] );
		add_filter( 'wp_grid_builder/frontend/localize_script', [ $this, 'unset_modules' ] );
		add_action( 'fl_builder_render_content_start', [ $this, 'get_modules' ] );

	}

	/**
	 * Whether to handle module
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param boolean $handle Whether to handle module.
	 * @param string  $module Module id.
	 * @return boolean
	 */
	public function handle_module( $handle, $module ) {

		if ( ! isset( $this->modules[ $module ] ) && ! wpgb_doing_ajax() ) {
			return false;
		}

		// Prevent rendering duplicate modules.
		unset( $this->modules[ $module ] );

		return true;

	}

	/**
	 * Loop through modules to render
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function get_modules() {

		// phpcs:ignore WordPress.Security.NonceVerification.Recommended
		if ( isset( $_GET['fl_builder'] ) || wpgb_doing_ajax() ) {
			return;
		}

		$modules = (array) \FLBuilderModel::get_nodes();

		foreach ( $modules as $module ) {

			if (
				empty( $module->node ) ||
				empty( $module->settings->grid ) ||
				empty( $module->settings->type ) ||
				'wpgb_facet' !== $module->settings->type

			) {
				continue;
			}

			$this->modules[ $module->settings->grid ] = $module->node;

		}
	}

	/**
	 * Unset shadow modules generated by Beaver Builder facets
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param array $data Holds data to localize.
	 * @return array
	 */
	public function unset_modules( $data ) {

		if ( empty( $data['shadowGrids'] ) ) {
			return $data;
		}

		$data['shadowGrids'] = array_values(
			array_filter(
				$data['shadowGrids'],
				function( $id ) {
					return strpos( $id, 'bb-module-' ) === false;
				}
			)
		);

		return $data;

	}

	/**
	 * Register script
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param array $scripts Holds script to register.
	 * @return array
	 */
	public function register_script( $scripts ) {

		// phpcs:ignore WordPress.Security.NonceVerification.Recommended
		if ( ! isset( $_GET['fl_builder'] ) ) {

			$scripts[] = [
				'handle'  => 'wpgb-beaver-builder',
				'source'  => WPGB_BEAVER_BUILDER_URL . 'assets/js/frontend.js',
				'version' => WPGB_BEAVER_BUILDER_VERSION,
			];
		}

		return $scripts;

	}

	/**
	 * Add inline javascript code to instantiate Beaver Builder modules
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param string $script Inline script.
	 * @return string
	 */
	public function inline_script( $script ) {

		$rendered = [];

		foreach ( Container::all() as $id => $instance ) {
			$rendered[] = $instance->inline_script( $script );
		}

		if ( ! empty( $rendered ) ) {
			$script .= implode( '', array_unique( $rendered ) );
		}

		return apply_filters( 'wp_grid_builder/template/inline_script', $script );

	}
}
