<?php
/**
 * Base
 *
 * @package   WP Grid Builder - Beaver Builder
 * @author    Loïc Blascos
 * @copyright 2019-2022 Loïc Blascos
 */

namespace WP_Grid_Builder_Beaver_Builder\Includes\Providers;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Providers base
 *
 * @class WP_Grid_Builder_Beaver_Builder\Includes\Providers\Base
 * @since 1.0.0
 */
abstract class Base {

	/**
	 * Constructor
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param array $module   Holds module attributes.
	 * @param array $settings Holds Request settings.
	 */
	public function __construct( $module, $settings = [] ) {

		$this->module   = $module;
		$this->settings = $settings;

	}

	/**
	 * Handle module before rendering
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function _before_render() { // phpcs:ignore PSR2.Methods.MethodDeclaration.Underscore

		add_filter( 'fl_builder_module_attributes', [ $this, 'inline_options' ], PHP_INT_MAX );
		add_filter( 'paginate_links', [ $this, 'paginate_links' ], PHP_INT_MAX );

		$this->before_render();

	}

	/**
	 * Handle module after rendering
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function _after_render() { // phpcs:ignore PSR2.Methods.MethodDeclaration.Underscore

		remove_filter( 'fl_builder_module_attributes', [ $this, 'inline_options' ], PHP_INT_MAX );
		remove_filter( 'paginate_links', [ $this, 'paginate_links' ], PHP_INT_MAX );

		$this->after_render();

	}

	/**
	 * Get module id
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return string
	 */
	public function get_id() {

		return $this->module->id;

	}

	/**
	 * Get post id
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return array
	 */
	public function get_post_id() {

		return $this->settings['post_id'];

	}

	/**
	 * Get CSS selector
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return string
	 */
	public function get_selector() {

		return '[data-options*="' . sanitize_html_class( $this->get_id() ) . '"]';

	}

	/**
	 * Inline module HTML data attribute options
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param array $attrs Holds module HTML attributes.
	 * @return array
	 */
	public function inline_options( $attrs ) {

		$attrs['data-options'] = esc_attr(
			wp_json_encode(
				wp_parse_args(
					[
						'id'            => $this->get_id(),
						'postId'        => $this->get_post_id(),
						'isTemplate'    => 'Beaver_Builder',
						'customContent' => true,
					],
					$this->options()
				)
			)
		);

		return $attrs;

	}

	/**
	 * Add inline javascript code to instantiate module
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param string $script Inline script.
	 * @return string
	 */
	public function inline_script( $script ) {

		return apply_filters(
			'wp_grid_builder_beaver_builder/inline_script',
			(
				'window.addEventListener(\'wpgb.loaded\',function(){' .
				'var template=document.querySelector(' . wp_json_encode( $this->get_selector() ) . ');' .
				'if(template){var wpgb=WP_Grid_Builder.instantiate(template);wpgb.init&&wpgb.init();' . $this->script() . '}});'
			)
		);
	}

	/**
	 * Replace paginated links with current permalink.
	 *
	 * @since 1.0.0
	 *
	 * @param string $link The paginated link URL.
	 * @return string
	 */
	public function paginate_links( $link ) {

		$query_string = wpgb_get_query_string();

		if ( empty( $query_string ) && ! wpgb_doing_ajax() ) {
			return $link;
		}

		if ( ! empty( $this->settings['permalink'] ) ) {

			$link = str_replace(
				trailingslashit( home_url( '', 'relative' ) ),
				trailingslashit( wp_make_link_relative( $this->settings['permalink'] ) ),
				$link
			);
		}

		$keys = preg_filter( '/^/', '_', array_keys( $query_string ) );
		$keys = array_merge( [ 'wpgb-ajax', 'action' ], $keys );
		$link = str_replace( '#038;', '&', $link );
		$link = remove_query_arg( $keys, $link );

		return $link;

	}

	/**
	 * Get module options to inline
	 *
	 * @since 1.0.0
	 * @access protected
	 */
	abstract protected function options();

	/**
	 * Get module script to localize
	 *
	 * @since 1.0.0
	 * @access protected
	 */
	abstract protected function script();

	/**
	 * Before module render
	 *
	 * @since 1.0.0
	 * @access protected
	 */
	abstract protected function before_render();

	/**
	 * After module render
	 *
	 * @since 1.0.0
	 * @access protected
	 */
	abstract protected function after_render();
}
