<?php
/**
 * Provider
 *
 * @package   WP Grid Builder - Beaver Builder
 * @author    Loïc Blascos
 * @copyright 2019-2022 Loïc Blascos
 */

namespace WP_Grid_Builder_Beaver_Builder\Includes\Providers;

use WP_Grid_Builder_Beaver_Builder\Includes;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Default provider
 *
 * @class WP_Grid_Builder_Beaver_Builder\Includes\Providers\Provider
 * @since 1.0.0
 */
class Provider extends Base {

	/**
	 * Module options
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return array
	 */
	public function options() {

		return [
			'itemSelector' => $this->get_selector() . ' > *',
			'isMainQuery'  => (
				empty( $this->module->data_source ) ||
				'main_query' === $this->module->data_source
			),
		];
	}

	/**
	 * Inline JS script
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return string
	 */
	public function script() {

		return '';

	}

	/**
	 * Apply hooks for current provider
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function before_render() {

		add_filter( 'fl_builder_loop_query_args', [ $this, 'set_query_args' ] );
		add_action( 'pre_get_posts', [ $this, 'main_query_args' ] );
		add_action( 'pre_get_posts', [ $this, 'unset_pagination_offset' ], PHP_INT_MAX - 8 );

		$this->main_query();
		$this->single_post();

	}

	/**
	 * Remove hooks for current provider
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function after_render() {

		remove_filter( 'fl_builder_loop_query_args', [ $this, 'set_query_args' ] );
		remove_action( 'pre_get_posts', [ $this, 'main_query_args' ] );
		remove_action( 'pre_get_posts', [ $this, 'unset_pagination_offset' ], PHP_INT_MAX - 8 );

	}

	/**
	 * Set query arguments to allow filtering
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param arrau $args Holds query arguments.
	 */
	public function set_query_args( $args ) {

		$args['wp_grid_builder']     = $this->get_id();
		$args['wpgb_beaver_builder'] = $this->get_id();

		if ( ! empty( $this->settings['lang'] ) ) {
			$args['lang'] = $this->settings['lang'];
		}

		// Reset offset from Beaver Builder pagination.
		$args['offset'] = empty( $this->module->offset ) ? 0 : (int) $this->module->offset;

		// Set page number from Beaver Builder pagination.
		if ( ! empty( $this->settings['paged'] ) ) {

			$args['paged'] = (int) $this->settings['paged'];
			set_query_var( 'paged', $args['paged'] );

		}

		// WP Grid Builder converts and removes the offset to avoid conflicts with pagination.
		$args['fl_original_offset'] = 0;

		return $args;

	}

	/**
	 * Set main query arguments
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param WP_Query $query The WP_Query instance.
	 */
	public function main_query_args( $query ) {

		if (
			! $query->get( 'wp_grid_builder' ) ||
			empty( $this->module->data_source ) ||
			'main_query' !== $this->module->data_source
		) {
			return;
		}

		$query->query_vars = $this->set_query_args( $query->query_vars );

	}

	/**
	 * Unset pagination offset after we filter content from WP Grid Builder
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param WP_Query $query The WP_Query instance.
	 */
	public function unset_pagination_offset( $query ) {

		if ( $this->get_id() !== $query->get( 'wpgb_beaver_builder' ) ) {
			return;
		}

		if ( empty( $this->settings['paged'] ) ) {
			return;
		}

		// We unset the offset to force WP_Query to use paged argument for the pagination.
		unset( $query->query_vars['offset'] );

	}

	/**
	 * Simulate main WordPress query
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function main_query() {

		global $wp_query;

		if ( ! wpgb_doing_ajax() ) {
			return;
		}

		if ( empty( $this->settings['main_query'] ) ) {
			return;
		}

		if (
			! empty( $this->module->data_source ) &&
			'main_query' !== $this->module->data_source
		) {
			return;
		}

		// We setup the main query custom argument.
		$this->settings['main_query']['wp_grid_builder'] = true;

		// We override the main query when filtering.
		$wp_query = new \WP_Query( $this->settings['main_query'] ); // phpcs:ignore WordPress.WP.GlobalVariablesOverride.Prohibited

	}

	/**
	 * Simulate single post page object
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function single_post() {

		global $post;

		if ( ! wpgb_doing_ajax() ) {
			return;
		}

		if ( ! empty( $this->settings['main_query'] ) ) {
			return;
		}

		$post_id = url_to_postid( wp_get_referer() );

		if ( empty( $post_id ) ) {
			return;
		}

		$post = get_post( $post_id ); // phpcs:ignore WordPress.WP.GlobalVariablesOverride.Prohibited

	}
}
