<?php
/**
 * WooCommerce
 *
 * @package   WP Grid Builder - Beaver Builder
 * @author    Loïc Blascos
 * @copyright 2019-2022 Loïc Blascos
 */

namespace WP_Grid_Builder_Beaver_Builder\Includes\Providers;

use WP_Grid_Builder_Beaver_Builder\Includes;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * WooCommerce module
 *
 * @class WP_Grid_Builder_Beaver_Builder\Includes\Providers\WooCommerce
 * @since 1.0.0
 */
final class WooCommerce extends Provider {

	/**
	 * Holds compatible WooCommerce shortcodes
	 *
	 * @since 1.0.0
	 * @access public
	 * @var array
	 */
	public $shortcodes = [
		'products',
		'product_category',
		'recent_products',
		'featured_products',
		'sale_products',
		'best_selling_products',
		'top_rated_products',
	];

	/**
	 * Module options
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return array
	 */
	public function options() {

		return [
			'itemSelector' => 'ul.products > li',
		];
	}

	/**
	 * Apply hooks for current provider
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function before_render() {

		add_action( 'pre_get_posts', [ $this, 'query_args' ] );

		foreach ( $this->shortcodes as $shortcode ) {

			add_action( 'woocommerce_shortcode_' . $shortcode . '_loop_no_results', [ $this, 'no_results' ] );
			add_filter( 'shortcode_atts_' . $shortcode, [ $this, 'unset_cache' ] );

		}
	}

	/**
	 * Remove hooks for current provider
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param string $content Module content.
	 */
	public function after_render( $content = '' ) {

		remove_action( 'pre_get_posts', [ $this, 'query_args' ] );

		foreach ( $this->shortcodes as $shortcode ) {

			remove_action( 'woocommerce_shortcode_' . $shortcode . '_loop_no_results', [ $this, 'no_results' ] );
			remove_filter( 'shortcode_atts_' . $shortcode, [ $this, 'unset_cache' ] );

		}
	}

	/**
	 * Unset WooCommerce products query cache to correctly filter
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param array $out The output array of shortcode attributes.
	 * @return array
	 */
	public function unset_cache( $out ) {

		$out['cache'] = false;

		return $out;

	}

	/**
	 * Set query arguments to allow filtering
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param WP_Query $query The WP_Query instance.
	 */
	public function query_args( $query ) {

		if ( ! in_array( 'product', (array) $query->get( 'post_type' ), true ) ) {
			return;
		}

		$query->set( 'wp_grid_builder', $this->get_id() );

		if ( ! empty( $this->settings['lang'] ) ) {
			$query->set( 'lang', $this->settings['lang'] );
		}

		if ( ! empty( $this->settings['paged'] ) ) {

			$query->set( 'paged', (int) $this->settings['paged'] );
			set_query_var( 'paged', $query->get( 'paged' ) );

		}

		if ( ! empty( $this->settings['orderby'] ) ) {

			// To keep order from WooCommerce sort list.
			$_GET['orderby'] = $this->settings['orderby'];
			( new \WC_Query() )->get_catalog_ordering_args();

		}

		remove_action( 'pre_get_posts', [ $this, 'query_args' ] );

	}

	/**
	 * Handle no results markup to properly restore unfiltered content
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param array $attributes Holds shortcode attributes.
	 */
	public function no_results( $attributes ) {

		if ( empty( $attributes['columns'] ) ) {
			$attributes['columns'] = 4;
		}

		echo '<ul class="products columns-' . (int) $attributes['columns'] . '"><li hidden></li></ul>';

	}
}
